package org.alfresco.bm.publicapi.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.FileableCmisObject;
import org.apache.chemistry.opencmis.client.api.Folder;
import org.apache.chemistry.opencmis.client.api.Property;
import org.apache.chemistry.opencmis.client.api.QueryResult;
import org.apache.chemistry.opencmis.client.api.Relationship;
import org.apache.chemistry.opencmis.client.api.Tree;
import org.apache.chemistry.opencmis.commons.PropertyIds;
import org.apache.chemistry.opencmis.commons.data.AllowableActions;
import org.apache.chemistry.opencmis.commons.data.PropertyData;
import org.apache.chemistry.opencmis.commons.definitions.FolderTypeDefinition;
import org.apache.chemistry.opencmis.commons.enums.Action;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Comment;
import org.springframework.social.alfresco.api.entities.Rating;
import org.springframework.social.alfresco.api.entities.Tag;

/**
 * An abstract base class for representing nodes.
 * 
 * @author steveglover
 *
 */
public abstract class Node
{
    protected String nodeId;
    protected Map<String, Object> properties;
    protected List<FolderNode> parents;
    protected Set<Action> allowableActions;
    protected AlfrescoList<Tag> tags;
    protected AlfrescoList<Comment> comments;
    protected AlfrescoList<Rating> ratings;

    protected Node()
    {
    	this.parents = new LinkedList<FolderNode>();
    }

    public AlfrescoList<Comment> getComments()
    {
        return comments;
    }

    public AlfrescoList<Rating> getRatings()
    {
        return ratings;
    }

    public void setComments(AlfrescoList<Comment> comments)
    {
        this.comments = comments;
    }

    public void setRatings(AlfrescoList<Rating> ratings)
    {
        this.ratings = ratings;
    }

    public AlfrescoList<Tag> getTags()
    {
        return tags;
    }

    public void setTags(AlfrescoList<Tag> tags)
    {
        this.tags = tags;
    }

    /**
     * Get allowable actions for the node.
     * 
     * @return the set of allowable (CMIS) actions on the node
     */
    public Set<Action> getAllowableActions()
    {
        return allowableActions;
    }
    
    /**
     * Set allowable CMIS actions for the node.
     * 
     * @param allowableActions
     */
    public void setAllowableActions(Set<Action> allowableActions)
    {
        this.allowableActions = allowableActions;
    }

    public String getRawNodeId()
    {
        return nodeId;
    }

    public List<FolderNode> getParents()
    {
        return parents;
    }

    public String getNodeId()
    {
    	// workaround for a bug in the public api
        String ret = nodeId;
    	int index = ret.indexOf(";pwc");
        if(index != -1)
        {
        	ret.substring(0, index);
        }
        return ret;
    }

    /**
     * Is this a folder node?
     * 
     * Override in subclasses.
     * 
     * @return is this a folder node?
     */
    public abstract boolean isFolder();

    /**
     * Set node id.
     * 
     * @param nodeId
     */
    public void setNodeId(String nodeId)
    {
        this.nodeId = nodeId;
    }

    /**
     * Set properties.
     * 
     * @param properties
     */
    public void setProperties(Map<String, Object> properties)
    {
        this.properties = properties;
    }
    
    public void setProperties(List<Property<?>> properties)
    {
	    for(Property<?> p : properties)
	    {
	        this.properties.put(p.getId(), p.getValue());
	    }
    }

    /**
     * Get all properties.
     */
    public Map<String, Object> getProperties()
    {
        return properties;
    }
    
    /**
     * Get a specific property.
     * 
     * @param name property name
     * @return get a property
     */
    public Object getProperty(String name)
    {
    	Object ret = null;

    	if(properties != null)
    	{
    		ret = properties.get(name);
    	}
    	
    	return ret;
    }
    
    /**
     * Create a Node from the CMIS QueryResult parameter.
     * 
     * @param qr query result
     * @return a node created from the CMIS query result
     */
    // TODO getFirstValue replace with getValues
    // how to determine type of result to choose Node or FolderNode?
    public static Node createNode(QueryResult qr)
    {
        Node n = null;

        String baseTypeId = (String)qr.getPropertyById(PropertyIds.BASE_TYPE_ID).getFirstValue();
        if(baseTypeId.equals("cmis:folder"))
        {
            n = new FolderNode();
        }
        else
        {
            n = new DocumentNode();
        }

        List<PropertyData<?>> props = qr.getProperties();
        Map<String, Object> properties = new HashMap<String, Object>();

        for(PropertyData<?> p : props)
        {
            properties.put(p.getId(), p.getFirstValue());
        }

        String nodeId = (String)qr.getPropertyById(PropertyIds.OBJECT_ID).getFirstValue();
        if(nodeId == null)
        {
            throw new RuntimeException("nodeId is null for " + qr);
        }
        n.setNodeId(nodeId);
        n.setProperties(properties);

        PropertyData<Object> prop = qr.getPropertyById(PropertyIds.PARENT_ID);
        if(prop != null)
        {
        	// TODO the parents won't have allowableActions, making it difficult to choose successor events
            String parentId = (String)prop.getFirstValue();
            List<FolderNode> parents = new ArrayList<FolderNode>(1);
            n.setParents(parents);
            FolderNode fn = new FolderNode();
            fn.setNodeId(parentId);
            parents.add(fn);
        }

        AllowableActions aa = qr.getAllowableActions();
        n.setAllowableActions(aa.getAllowableActions());
        
        return n;
    }
    
    public void setParents(List<FolderNode> parents)
    {
    	this.parents.addAll(parents);
    }

    /**
     * Create a Node and descendants from the CMIS tree parameter.
     * 
     * @param tree CMIS tree object
     * @return a node created from the CMIS tree
     */
    public static Node createNode(Tree<FileableCmisObject> tree)
    {
        FileableCmisObject fco = tree.getItem();
        Node descendant = Node.createNode(fco);
        for(Tree<FileableCmisObject> child : tree.getChildren())
        {
            Node childNode = Node.createNode(child);
            // it must be a folder
            ((FolderNode)descendant).addChild(childNode);
        }
        return descendant;
    }

    /**
     * Create a Node from the CMIS CmisObject parameter.
     * 
     * @param o CMISobject
     * @return a node created from the CMIS object
     */
    public static Node createNode(CmisObject o)
    {
        if(o instanceof FileableCmisObject)
        {
            return createNode((FileableCmisObject)o);
        }
        else if(o instanceof Relationship)
        {
            // TODO revisit: CMIS Can't create a relationship node
            throw new IllegalArgumentException("Can't create a relationship node");
        }
        else
        {
            throw new IllegalArgumentException();
        }
    }

    /**
     * Create a Node from the CMIS FileableCmisObject parameter.
     * 
     * @param fco
     * @return a node created from the CMIS object
     */
    public static Node createNode(FileableCmisObject fco)
    {
        Node n = null;
        if(fco.getBaseType() instanceof FolderTypeDefinition)
        {
            n = new FolderNode();
        }
        else
        {
            n = new DocumentNode();
        }

        String nodeId = fco.getId();
        if(nodeId == null)
        {
            throw new RuntimeException("nodeId is null for " + fco);
        }
        n.setNodeId(nodeId);

        Map<String, Object> properties = new HashMap<String, Object>();
        for(Property<?> p : fco.getProperties())
        {
            properties.put(p.getId(), p.getValue());
        }
        n.setProperties(properties);
        
        List<Folder> parents = fco.getParents();
        List<FolderNode> parentNodes = new ArrayList<FolderNode>(parents.size());
    	for(Folder folder : parents)
    	{
    		parentNodes.add((FolderNode)Node.createNode(folder));
    	}
        n.setParents(parentNodes);
        
        AllowableActions aa = fco.getAllowableActions();
        n.setAllowableActions(aa.getAllowableActions());
        
        return n;
    }

    @Override
    public String toString()
    {
        return "Node [nodeId=" + nodeId + ", properties=" + properties + "]";
    }
}
