package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.site.SiteMember;
import org.alfresco.bm.site.SiteRole;
import org.springframework.http.HttpStatus;
import org.springframework.social.alfresco.api.Alfresco;
import org.springframework.social.alfresco.api.entities.Role;
import org.springframework.social.alfresco.connect.exception.AlfrescoException;

/**
 * Create a site member.
 * 
 * @author steveglover
 *
 */
public class BasicCreateSiteMember extends AbstractEventProcessor
{
    private String eventNameSiteMemberCreated = "siteMemberCreated";

    private Services services;
    private PublicApiFactory publicApiFactory;

    /**
     * @param services                     access to collection services
     * @param publicApiFactory             a factory for creating connections to the public api
     * @param eventNameSiteMemberCreated   the site member created event
     */
    public BasicCreateSiteMember(Services services, PublicApiFactory publicApiFactory, String eventNameSiteMemberCreated)
    {
        super();
        this.services = services;
        this.publicApiFactory = publicApiFactory; 
        this.eventNameSiteMemberCreated = eventNameSiteMemberCreated;
    }

    /**
     * @param services                     access to collection services
     * @param publicApiFactory             a factory for creating connections to the public api
     */
    public BasicCreateSiteMember(Services services, PublicApiFactory publicApiFactory)
    {
        super();
        this.services = services;
        this.publicApiFactory = publicApiFactory; 
    }
    
    private Alfresco getPublicApi(String userId) throws Exception
    {
        Alfresco alfresco = publicApiFactory.getPublicApi(userId);
        return alfresco;
    }
    
    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        CreateSiteMemberRequest createSiteMemberRequest = (CreateSiteMemberRequest)event.getDataObject();
        String userId = createSiteMemberRequest.getRunAsUserId();
        String networkId = createSiteMemberRequest.getNetworkId();
        String siteId = createSiteMemberRequest.getSiteId();
        String personId = createSiteMemberRequest.getUserId();
        Role role = createSiteMemberRequest.getRole();
        EventProcessorResponse response = null;

        List<Event> nextEvents = new ArrayList<Event>();
        boolean success = false;
        String msg = null;

        // Check the input
        if (siteId == null || personId == null || role == null || userId == null || networkId == null)
        {
             msg = "Invalid site member request: " + createSiteMemberRequest;
             return new EventResult(msg, nextEvents, false);
        }

        SiteMember siteMember = new SiteMember(personId, siteId, null, SiteRole.valueOf(role.toString()));
        try
        {
            getPublicApi(userId).addMember(networkId, siteId, personId, role);
            services.getSiteDataService().setSiteMemberCreated(siteMember.getSiteId(), siteMember.getUsername(), siteMember.getRole(), Boolean.TRUE);
            
            response = new EventProcessorResponse("Added site member", true, siteMember);
            
            msg = "Created site member: \n" + "   Response: " + response;
            success = true;
            nextEvents.add(new Event(eventNameSiteMemberCreated, 0L, response));
        }
        catch(AlfrescoException e)
        {
            if(e.getStatusCode().equals(HttpStatus.CONFLICT))
            {
                // already a member
                services.getSiteDataService().setSiteMemberCreated(siteMember.getSiteId(), siteMember.getUsername(), siteMember.getRole(), Boolean.TRUE);
                
                response = new EventProcessorResponse("Added site member", true, siteMember);
                
                msg = "Created site member: \n" + "   Response: " + response;
                success = true;
                nextEvents.add(new Event(eventNameSiteMemberCreated, 0L, response));
            }
            else
            {
                throw e;
            }
        }

        if (logger.isDebugEnabled())
        {
            logger.debug(msg);
        }

        EventResult result = new EventResult(
                msg,
                nextEvents,
                success);
        return result;
    }
}
