package org.alfresco.bm.publicapi.process;

import java.io.Serializable;
import java.util.Map;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.CreateDocumentRequestFactory;
import org.alfresco.bm.publicapi.data.DocumentContent;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Path;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Folder;
import org.apache.chemistry.opencmis.client.api.Session;
import org.apache.chemistry.opencmis.commons.PropertyIds;
import org.apache.chemistry.opencmis.commons.data.ContentStream;
import org.apache.chemistry.opencmis.commons.enums.VersioningState;

/**
 * Create a document.
 * 
 * @author steveglover
 *
 */
public class CreateDocument extends AbstractRepositoryEventSelectorProcessor
{
	private CreateDocumentRequestFactory requestFactory;
	
    /**
     * 
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateDocument(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector, CreateDocumentRequestFactory requestFactory)
    {
        super(services, publicApiFactory, eventSelector);
        this.requestFactory = requestFactory;
    }

    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        CreateDocumentByParentIdRequest ret = null;

        if(input != null)
        {
            Request request = (Request)input;
            
	        FolderNode parentFolder = getFolder(request, resp, canCreateDocumentMatcher);
	        if(parentFolder != null)
	        {
	            logger.debug("CreateDocument.createDataObject parentFolder = " + parentFolder);
	            ret = requestFactory.create(request.getNetworkId(), request.getRunAsUserId(), parentFolder);
	        }
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to create document, input is null", true, null);
        }
        else
        {
            Request request = (Request)input;

            String userId = null;
            String networkId = null;
            Path parentFolderPath = null;
            FolderNode parentFolder = null;
            VersioningState versioningState = null;
            DocumentContent content = null;
            String type = null;
            Map<String, Serializable> properties = null;
            
	        if(request instanceof CreateDocumentByParentPathRequest)
	        {
	            CreateDocumentByParentPathRequest createDocumentRequest = (CreateDocumentByParentPathRequest)request;
	            userId = createDocumentRequest.getRunAsUserId();
	            networkId = createDocumentRequest.getNetworkId();
	            parentFolderPath = createDocumentRequest.getParentFolderPath();
	            versioningState = createDocumentRequest.getVersioningState();
	            content = createDocumentRequest.getContent();
	            type = createDocumentRequest.getType();
	            properties = createDocumentRequest.getProperties();
	        }
	        else if(request instanceof CreateDocumentByParentIdRequest)
	        {
	            CreateDocumentByParentIdRequest createDocumentRequest = (CreateDocumentByParentIdRequest)request;
	            userId = createDocumentRequest.getRunAsUserId();
	            networkId = createDocumentRequest.getNetworkId();
	            parentFolder = createDocumentRequest.getParentFolder();
	            versioningState = createDocumentRequest.getVersioningState();
	            content = createDocumentRequest.getContent();
	            type = createDocumentRequest.getType();
	            properties = createDocumentRequest.getProperties();
	        }
	        else
	        {
	            throw new IllegalArgumentException("Invalid request: " + request);
	        }
	        
	        if(parentFolder == null && parentFolderPath == null)
	        {
	            response = new EventProcessorResponse("Must provide either parent folder or parent folder path", false, null);
	        }
	        else if(userId == null || networkId == null)
	        {
	            response = new EventProcessorResponse("Must provide run as user id and network id", false, null);
	        }
	        else
	        {
	            properties.put(PropertyIds.OBJECT_TYPE_ID, type);
	
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            Folder parent = null;
	            if(parentFolder != null)
	            {
	                parent = (Folder)cmisSession.getObject(parentFolder.getNodeId());
	            }
	            else if(parentFolderPath != null)
	            {
	                parent = (Folder)cmisSession.getObjectByPath(parentFolderPath.getPath());
	            }
	            ContentStream contentStream = content.getContentStream();
	
	            Document doc = parent.createDocument(properties, contentStream, versioningState);
	            DocumentNode res = (DocumentNode)Node.createNode(doc);
	            response = new EventProcessorResponse("Created document '" + res.getNodeId(), true, res, true);
	        }
        }

        return response;
    }
}
