package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;

/**
 * Creates public api sessions by selecting a random user from the users collection and then
 * generating the initial public api request for that user.
 * 
 * Currently, "sessions" are created "in-domain" i.e. for user's in their home domain
 * due to a restriction of PrepareSiteMembers.
 * 
 * @author steveglover
 *
 */
// TODO constraints on which networks and users to use, perhaps a minimum threshold on number of users in the network,
// number of sites in the network etc?
public class CreateSessions extends AbstractEventProcessor
{
    private static final int DEFAULT_BATCH_SIZE = 10;
    private static final long DEFAULT_EXPECTED_CREATE_TIME = 20;

    private int numberOfSessions;
    private int batchSize;
    
    /**
     * Creates a new instance.
     * 
     * @param services                access to collection services
     * @param numberOfSessions        a running count of the remaining number of sessions to create
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public CreateSessions(int numberOfSessions)
    {
        super();
        this.numberOfSessions = numberOfSessions;
        this.batchSize = DEFAULT_BATCH_SIZE;
    }

    /**
     * Override the {@link #DEFAULT_BATCH_SIZE default} batch size
     */
    public void setBatchSize(int batchSize)
    {
        this.batchSize = batchSize;
    }

    public EventResult processEvent(Event event) throws Exception
    {
        String msg = null;
        Integer sessionsCount = (Integer)event.getDataObject();
        if(sessionsCount == null)
        {
            sessionsCount = numberOfSessions;
        }
        List<Event> nextEvents = new ArrayList<Event>();
        int remainingSessionsToCreate = sessionsCount - batchSize;
        int sessionsCreated = 0;

        long lastEventTime = System.currentTimeMillis();
        for (int i = 0; i < Math.min(batchSize, sessionsCount); i++)
        {
        	Event initialPublicApiEvent = new Event("publicapi.createSession", System.currentTimeMillis(), null);
        	nextEvents.add(initialPublicApiEvent);
        	sessionsCreated++;
        }

        if(sessionsCreated > 0)
        {
        	msg = "Created " + sessionsCreated + " public api session(s)";
	        if (remainingSessionsToCreate > 0)
	        {
	            // Reschedule for the next batch (might be zero next time) only if there are more
	        	// sessions to create and sessions were created this time round (if they were not,
	        	// indicates there was an issue selecting users for the sessions, so best to
	        	// bail out)
	            Event self = new Event("publicapi.createSessions", System.currentTimeMillis(), remainingSessionsToCreate);
	            self.setScheduledTime(lastEventTime + DEFAULT_EXPECTED_CREATE_TIME);
	            nextEvents.add(self);
	        }
        }
        else
        {
        	msg = "No sessions created, are there any users created?";
        }

        // Return messages + next events
        return new EventResult(msg, nextEvents);
    }
}