package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.alfresco.bm.site.SiteData;
import org.springframework.social.alfresco.api.Alfresco;
import org.springframework.social.alfresco.api.entities.LegacySite;
import org.springframework.social.alfresco.api.entities.Site.Visibility;
import org.springframework.social.alfresco.connect.exception.AlfrescoException;

public class CreateSite extends AbstractEventProcessor
{
    private String eventNameSiteCreated = "siteCreated";

    private Services services;
    private PublicApiFactory publicApiFactory;

    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventNameSiteCreated    the site created event name
     */
    public CreateSite(Services services, PublicApiFactory publicApiFactory, String eventNameSiteCreated)
    {
        super();
        this.services = services;
        this.publicApiFactory = publicApiFactory; 
        this.eventNameSiteCreated = eventNameSiteCreated;
    }

    public CreateSite(Services services, PublicApiFactory publicApiFactory)
    {
        super();
        this.services = services;
        this.publicApiFactory = publicApiFactory; 
    }

    private Alfresco getPublicApi(String userId) throws Exception
    {
        Alfresco alfresco = publicApiFactory.getPublicApi(userId);
        return alfresco;
    }
    
    private void markSiteCreated(String siteId)
    {
        // Create site has succeeded.  Mark the site.
        boolean created = services.getSiteDataService().setSiteCreated(siteId, Boolean.TRUE);
        if(!created)
        {
            throw new RuntimeException("Site created, but unable to mark it so in Mongo");
        }
    }
    
    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        CreateSiteRequest request = (CreateSiteRequest)event.getDataObject();
        SiteData site = request.getSite();
        String userId = request.getRunAsUserId();
        String networkId = request.getNetworkId();

        List<Event> nextEvents = new ArrayList<Event>();
        boolean success = false;
        String msg = null;

        if(site != null && userId != null && networkId != null)
        {
        	try
        	{
	            Alfresco publicApi = getPublicApi(userId);
	            LegacySite ret = publicApi.createSite(networkId, site.getSiteId(), site.getSitePreset(), site.getTitle(),
	                    site.getDescription(), Visibility.valueOf(site.getVisibility().toString()));
	            
	            // TODO use ret
	
	            // Create site has succeeded.  Mark the site.
	            markSiteCreated(site.getSiteId());
	
	            msg = "Created site: \n" + "   Response: " + site;
	            success = true;
	            nextEvents.add(new Event(eventNameSiteCreated, 0L, ret));
        	}
        	catch(AlfrescoException e)
        	{
        		// not an ideal way to detected this but the Alfresco web script doesn't make it easy
        		String errMsg = e.getMessage();
        		if(errMsg.indexOf("error.duplicateShortName") != -1)
        		{
        			// already exists
    	            // Create site has succeeded.  Mark the site.
    	            markSiteCreated(site.getSiteId());

    	            msg = "Site " + site + " exists";
    	            success = true;
        		}
        		else
        		{
        			throw e;
        		}
        	}
        }
        else
        {
            msg = "Skipping create site: no site supplied.";
        }

        if (logger.isDebugEnabled())
        {
            logger.debug(msg);
        }

        EventResult result = new EventResult(
                msg,
                nextEvents,
                success);
        return result;
    }
}
