package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.site.SiteData;
import org.alfresco.bm.site.SiteMember;
import org.alfresco.bm.site.SiteRole;
import org.springframework.social.alfresco.api.entities.Role;

/**
 * Generate create site member events for site members in the site members collection that are pending creation.
 */
public class CreateSiteMembers extends AbstractEventProcessor
{
    private static final int DEFAULT_BATCH_SIZE = 100;
    private static final long DEFAULT_EXPECTED_CREATE_TIME = 20;

    private String eventNameSiteMembersCreated = "siteMembersCreated";
    private String eventNameCreateSiteMember = "createSiteMember";
    private String eventNameCreateSiteMembers = "createSiteMembers";

    private int batchSize = DEFAULT_BATCH_SIZE;

    private Services services;

    /**
     * @param services                     access to collection services
     * @param eventNameSiteMemberCreated   the site member created event name
     */
    public CreateSiteMembers(Services services, String eventNameSiteMemberCreated)
    {
        super();
        this.services = services;
        this.eventNameSiteMembersCreated = eventNameSiteMemberCreated;
    }

    public CreateSiteMembers(Services services)
    {
        super();
        this.services = services;
    }

    @Override
    public EventResult processEvent(Event event) throws Exception
    {
        List<Event> nextEvents = new ArrayList<Event>();
        
        // Schedule events for each site member to be created
        int numSitesMembers = 0;

        List<SiteMember> pendingSiteMembers = services.getSiteDataService().getSiteMembersPendingCreation(0, batchSize);
        if(pendingSiteMembers.size() == 0)
        {
            // There is nothing more to do
            Event doneEvent = new Event(eventNameSiteMembersCreated, System.currentTimeMillis(), null);
            nextEvents.add(doneEvent);
        }
        else
        {
            long lastEventTime = System.currentTimeMillis();
            for (SiteMember siteMember : pendingSiteMembers)
            {
                lastEventTime = System.currentTimeMillis();
                // Do we need to schedule it?
                String siteId = siteMember.getSiteId();
                SiteData site = services.getSiteDataService().findSiteBySiteId(siteId);
                if(site != null)
                {
	                String userId = siteMember.getUsername();
	                SiteRole role = siteMember.getRole();

	                CreateSiteMemberRequest request = new CreateSiteMemberRequest(site.getNetworkId(), site.getCreatedBy(), siteId, userId, Role.valueOf(role.toString()));
	                Event nextEvent = new Event(eventNameCreateSiteMember, System.currentTimeMillis(), request);
	                nextEvents.add(nextEvent);
	                numSitesMembers++;
                }
                else
                {
                	// TODO
                }
            }

            // Reschedule for the next batch (might be zero next time)
            Event self = new Event(eventNameCreateSiteMembers, System.currentTimeMillis(), null);
            self.setScheduledTime(lastEventTime + DEFAULT_EXPECTED_CREATE_TIME);
            nextEvents.add(self);
        }

        // Return messages + next events
        return new EventResult("Scheduled " + numSitesMembers + " site member(s) for creation", nextEvents);
    }
}
