package org.alfresco.bm.publicapi.process;

import java.util.ArrayList;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.AbstractEventProcessor;
import org.alfresco.bm.event.Event;
import org.alfresco.bm.event.EventResult;
import org.alfresco.bm.site.SiteData;
import org.alfresco.bm.site.SiteDataService;

/**
 * Generate create site events for sites in the sites collection that are pending creation.
 * 
 * @author steveglover
 *
 */
public class CreateSites extends AbstractEventProcessor
{
    private static final int DEFAULT_BATCH_SIZE = 100;
    private static final long DEFAULT_EXPECTED_CREATE_TIME = 20;
    
    public static final String EVENT_NAME_CREATE_SITE = "createSite";
    public static final String EVENT_NAME_CREATE_SITES = "createSites";
    public static final String EVENT_NAME_SITES_CREATED = "sitesCreated";
    
    private SiteDataService siteDataService;
    private int batchSize;
    private String eventNameCreateSites;
    private String eventNameSitesCreated;

    /**
     * Creates a new instance.
     * 
     * @param services                access to collection services
     */
    public CreateSites(Services services)
    {
        this.siteDataService = services.getSiteDataService();
        
        batchSize = DEFAULT_BATCH_SIZE;
        eventNameCreateSites = EVENT_NAME_CREATE_SITES;
        eventNameSitesCreated = EVENT_NAME_SITES_CREATED;
    }

    /**
     * Override the {@link #DEFAULT_BATCH_SIZE default} batch size
     */
    public void setBatchSize(int batchSize)
    {
        this.batchSize = batchSize;
    }
    
    /**
     * Override the {@link #EVENT_NAME_CREATE_SITES default} event name for more
     * user creation scheduling
     */
    public void setEventNameCreateSites(String eventNameCreateSites)
    {
        this.eventNameCreateSites = eventNameCreateSites;
    }

    /**
     * Override the {@link #EVENT_NAME_SITES_CREATED default} event name
     * indicating that enough users have been created
     */
    public void setEventNameSitesCreated(String eventNameSitesCreated)
    {
        this.eventNameSitesCreated = eventNameSitesCreated;
    }

    public EventResult processEvent(Event event) throws Exception
    {
        List<Event> nextEvents = new ArrayList<Event>();
        
        // Schedule events for each site to be created
        int numSites = 0;
        int index = 0;
        List<SiteData> pendingSites = siteDataService.getSitesPendingCreation(index, batchSize);
        if(pendingSites.size() == 0)
        {
            // There is nothing more to do
            Event doneEvent = new Event(eventNameSitesCreated, System.currentTimeMillis(), null);
            nextEvents.add(doneEvent);
        }
        else
        {
            long lastEventTime = System.currentTimeMillis();
            for (SiteData site : pendingSites)
            {
                lastEventTime = System.currentTimeMillis();
                // Do we need to schedule it?
                String createdBy = site.getCreatedBy();
                String networkId = site.getNetworkId();

                CreateSiteRequest request = new CreateSiteRequest(networkId, createdBy, site);
                Event nextEvent = new Event(EVENT_NAME_CREATE_SITE, System.currentTimeMillis(), request);
                nextEvents.add(nextEvent);
                numSites++;
            }

            // Reschedule for the next batch (might be zero next time)
            Event self = new Event(eventNameCreateSites, System.currentTimeMillis(), null);
            self.setScheduledTime(lastEventTime + DEFAULT_EXPECTED_CREATE_TIME);
            nextEvents.add(self);
        }
        
        // Return messages + next events
        return new EventResult("Scheduled " + numSites + " site(s) for creation", nextEvents);
    }
}
