package org.alfresco.bm.publicapi.process;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.DocumentNode;
import org.alfresco.bm.publicapi.data.FolderNode;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.ItemIterable;
import org.apache.chemistry.opencmis.client.api.Property;
import org.apache.chemistry.opencmis.commons.definitions.FolderTypeDefinition;
import org.apache.chemistry.opencmis.commons.enums.IncludeRelationships;

/**
 * Get children event processor.
 * 
 * @author steveglover
 *
 */
public class GetChildren extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public GetChildren(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Unable to get children, input is null", true, null);
        }
        else
        {
	        GetChildrenRequest getChildrenRequest = (GetChildrenRequest)input;
	        String userId = getChildrenRequest.getRunAsUserId();
	        String networkId = getChildrenRequest.getNetworkId();
	        FolderNode parent = getChildrenRequest.getFolder();
	        IncludeRelationships includeRelationships = getChildrenRequest.getIncludeRelationships();
	        if(includeRelationships == null)
	        {
	            includeRelationships = IncludeRelationships.NONE;
	        }
	        Boolean includeAcls = getChildrenRequest.getIncludeAcls();
	        if(includeAcls == null)
	        {
	            includeAcls = Boolean.FALSE;
	        }
	        Set<String> propertyFilter = getChildrenRequest.getPropertyFilter();
	        Boolean includePolicies = getChildrenRequest.getIncludePolicies();
	        if(includePolicies != null)
	        {
	            includePolicies = Boolean.FALSE;
	        }
	
	        if(parent != null && userId != null && networkId != null)
	        {
	            ItemIterable<CmisObject> children = getPublicApi(userId).getChildren(networkId, parent.getNodeId(), 0, Integer.MAX_VALUE,
	                    includeRelationships, includeAcls, propertyFilter, includePolicies);
	            if(children != null)
	            {
	                for(CmisObject c : children)
	                {
	                    Node child = null;
	                    if(c.getBaseType() instanceof FolderTypeDefinition)
	                    {
	                        child = new FolderNode();
	                        parent.addFolder((FolderNode)child);
	                    }
	                    else
	                    {
	                        child = new DocumentNode();
	                        parent.addDocument((DocumentNode)child);
	                    }
	
	                    String nodeId = c.getId();
	                    if(nodeId == null)
	                    {
	                        throw new RuntimeException("nodeId is null for " + c);
	                    }
	
	                    child.setNodeId(nodeId);
	                    Map<String, Object> properties = new HashMap<String, Object>();
	                    child.setProperties(properties);
	                    for(Property<?> p : c.getProperties())
	                    {
	                        properties.put(p.getId(), p.getValueAsString());
	                    }
	                }
	
	                response = new EventProcessorResponse("Got children for '" + parent, true, parent, true);
	            }
	            else
	            {
	                response = new EventProcessorResponse("Get children empty for " + parent, false, null);
	            }
	        }
	        else
	        {
	            response = new EventProcessorResponse("Skipping get children: no folderId supplied.", false, null);
	        }
        }

        return response;
    }
    
    @Override
    public Object createDataObject(Object input, Object resp) throws Exception
    {
        GetChildrenRequest ret = null;
        
        if(input != null)
        {
            Request request = (Request)input;

	        FolderNode folder = getFolder(request, resp, canGetChildrenMatcher);
	        if(folder != null)
	        {
	            logger.debug("GetChildren.createDataObject folder = " + folder);
	
	            Integer depth = null;
	            IncludeRelationships includeRelationships = IncludeRelationships.BOTH;
	            Boolean includeAcls = true;
	            Set<String> propertyFilter = null;
	            Boolean includePolicies = true;
	            String networkId = request.getNetworkId();
	            String userId = request.getRunAsUserId();
	    
	            ret = new GetChildrenRequest(networkId, userId, folder, depth, includeRelationships, includeAcls, propertyFilter, includePolicies);
	        }
	        else
	        {
	        	ret = null;
	        }
        }

        return ret;
    }
}
