package org.alfresco.bm.publicapi.process;

import java.text.MessageFormat;
import java.util.List;

import org.alfresco.bm.Services;
import org.alfresco.bm.event.selector.EventProcessorResponse;
import org.alfresco.bm.event.selector.EventSelector;
import org.alfresco.bm.publicapi.AbstractRepositoryEventSelectorProcessor;
import org.alfresco.bm.publicapi.data.Node;
import org.alfresco.bm.publicapi.data.NodesContainer;
import org.alfresco.bm.publicapi.data.Request;
import org.alfresco.bm.publicapi.factory.PublicApiFactory;
import org.apache.chemistry.opencmis.client.api.CmisObject;
import org.apache.chemistry.opencmis.client.api.ItemIterable;
import org.apache.chemistry.opencmis.client.api.QueryResult;
import org.apache.chemistry.opencmis.client.api.Session;
import org.springframework.social.alfresco.api.entities.AlfrescoList;
import org.springframework.social.alfresco.api.entities.Site;

/**
 * Executes a CMIS query against the public api.
 * 
 * @author steveglover
 *
 */
// TODO construct query to search for strings in a previous getContent call
public class Query extends AbstractRepositoryEventSelectorProcessor
{
    /**
     * @param services                access to collection services
     * @param publicApiFactory        a factory for creating connections to the public api
     * @param eventSelector           the event selector to use in selecting the next event
     */
    public Query(Services services, PublicApiFactory publicApiFactory, EventSelector eventSelector)
    {
        super(services, publicApiFactory, eventSelector);
    }

	private String randomFolderSearch(CmisObject parentFolder)
	{
		String statement = null;

        if(parentFolder != null)
        {
        	String objectId = parentFolder.getId();
        	statement = MessageFormat.format("SELECT * FROM cmis:folder as f where IN_TREE(f, ''{0}'')",
        				objectId);
        }
        else
        {
        	statement = "SELECT * FROM cmis:folder";
        }

        return statement;
	}

	private String randomDocumentSearch(CmisObject parentFolder, String searchString)
	{
		String statement = null;

        if(parentFolder != null)
        {
        	String objectId = parentFolder.getId();
        	if(searchString != null)
        	{
        		statement = MessageFormat.format("SELECT * FROM cmis:document as d where IN_TREE(d, ''{0}'') AND contains(''{1}'')",
        				objectId, searchString);
        	}
        	else
        	{
        		statement = MessageFormat.format("SELECT * FROM cmis:document as d where IN_TREE(d, ''{0}'')",
        				objectId);
        	}
        }
        else
        {
        	if(searchString != null)
        	{
        		statement = MessageFormat.format("SELECT * FROM cmis:document as d where contains(''{1}'')",
        				searchString);
        	}
        	else
        	{
        		statement = "SELECT * FROM cmis:document";
        	}
        }

        return statement;
	}

    @Override
    public Object createDataObject(Object input, Object response) throws Exception
    {
    	QueryRequest ret = null;

        if(input != null)
        {
	        Request request = (Request)input;
	        String networkId = request.getNetworkId();
	        String userId = request.getRunAsUserId();
	        int skipCount = 0;
	        int maxItems = 20;
	        String statement = null;

	        if(input instanceof QueryRequest)
	        {
	        	// if previous request was a query, reuse it but skip to the next page
	        	QueryRequest queryRequest = (QueryRequest)input;
	        	skipCount = queryRequest.getSkipCount() + queryRequest.getMaxItems();
	        	statement = queryRequest.getStatement();
	        }
	        else
	        {
	        	StringBuilder path = new StringBuilder("/Sites/");

	        	if(response instanceof Site)
	        	{
	        		// search under the documentLibrary
	        		Site site = (Site)response;
	        		path.append(site.getId());
	        		path.append("/documentLibrary");
	        	}
	        	else if(response instanceof AlfrescoList<?>)
	        	{
	        		AlfrescoList<?> aList = (AlfrescoList<?>)response;
	        		Site site = findSite(request, aList);
	        		if(site != null)
	        		{
		        		// search under the documentLibrary
		        		path.append(site.getId());
		        		path.append("/documentLibrary");
	        		}
	        	}
	        	else if(response instanceof List<?>)
	        	{
	        		List<?> aList = (List<?>)response;
	        		Site site = findSite(request, aList);
	        		if(site != null)
	        		{
		        		// search under the documentLibrary
		        		path.append(site.getId());
		        		path.append("/documentLibrary");
	        		}
	        	}

		        // Try to get the object id of the Sites node, so that we can query under Sites
		        Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
		        CmisObject parentFolder = cmisSession.getObjectByPath(path.toString());

		        int idx = random.nextInt(2);
		        statement = (idx == 0 ? randomDocumentSearch(parentFolder, randomSearchString()) : randomFolderSearch(parentFolder));
	        }

	        logger.debug("Query.createDataObject statement = " + statement);

	        ret = new QueryRequest(networkId, userId, statement, skipCount, maxItems);
        }

        return ret;
    }

    @Override
    protected EventProcessorResponse processEventImpl(Object input) throws Exception
    {
        EventProcessorResponse response = null;

        if(input == null)
        {
        	 response = new EventProcessorResponse("Skipping query execution, input is null", true, null);
        }
        else
        {
	        QueryRequest queryRequest = (QueryRequest)input;
	        String userId = queryRequest.getRunAsUserId();
	        String networkId = queryRequest.getNetworkId();
	        String statement = queryRequest.getStatement();
	        Integer skipCount = queryRequest.getSkipCount();
	        Integer maxItems = queryRequest.getMaxItems();
	
	        if(userId == null)
	        {
	            response = new EventProcessorResponse("Skipping query execution, must provide a userId", false, null);
	        }
	        else if(networkId == null)
	        {
	            response = new EventProcessorResponse("Skipping query execution, must provide a userId", false, null);
	        }
	        else if(statement == null)
	        {
	            response = new EventProcessorResponse("Skipping query execution, must provide a query", false, null);
	        }
	        else
	        {
	            Session cmisSession = getPublicApi(userId).getCMISSession(networkId);
	            ItemIterable<QueryResult> results = cmisSession.query(statement, false);
	            ItemIterable<QueryResult> page = results.skipTo(skipCount).getPage(maxItems);
	
	            // convert to a NodeContainer, to make it easier to select random folders and documents from the results
	            NodesContainer nodes = new NodesContainer();
	            for(QueryResult qr : page)
	            {
	                nodes.addNode(Node.createNode(qr));
	            }
	            response = new EventProcessorResponse("Executed query '" + statement + "' : " + nodes.getDocumentNodes().size() + " documents, "
	                    + nodes.getFolderNodes().size() + " folders", true, nodes, true);
	        }
        }

        return response;
    }
}
